/*
Copyright (c) 2006-2007, Tom Thielicke IT Solutions

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

/****************************************************************
**
** Implementation of the KeyboardSql class
** File name: keyboardsql.cpp
**
****************************************************************/

#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>
#include <QFile>
#include <QTextStream>
#include <QString>

#include "keyboardsql.h"

KeyboardSql::KeyboardSql(QString op) {
	// Set the keyboard layout
	opSystem = op;
}

bool KeyboardSql::getKeyLayout(QChar givenchar, int *left, int *top,
	int *color, int *form, int *modifier, int *modifier2, int *finger,
	QString *status, bool isQwerty) {

	// Convert given char into properties of a key
	QString tableName = "key_layouts";
	if (isQwerty) {
		tableName = "key_layouts_us";
	}
	QSqlQuery query;
	if (query.exec("SELECT " + tableName +
		".left, " + tableName + ".top, "
		"" + tableName + ".color, " + tableName +
		".form, character_list_" + opSystem +
		".modifier, character_list_" + opSystem +
		".modifier2, " + tableName + ".finger, " + tableName +
		".status_text "
		"FROM character_list_" + opSystem +
		", " + tableName + " WHERE " + tableName +
		".key_id = character_list_" + opSystem +
		".layout AND character_list_" + opSystem + ".unicode = " +
		QString::number(givenchar.unicode()) + ";")) {
		if (query.first()) {
			*left = query.value(0).toInt();
			*top = query.value(1).toInt();
			*color = query.value(2).toInt();
			*form = query.value(3).toInt();
			*modifier = query.value(4).toInt();
			*modifier2 = query.value(5).toInt();
			*finger = query.value(6).toInt();
			*status = query.value(7).toString();
			return true;
		}
	}
	return false;
}

bool KeyboardSql::getModifierLayout(int givenmodifier, int *left, int *top,
	int *color, int *form, int *finger, QString *status, bool isQwerty) {

	// Convert given id into properties of a key
	QString tableName = "key_layouts";
	if (isQwerty) {
		tableName = "key_layouts_us";
	}
	QSqlQuery query;
	if (query.exec("SELECT " + tableName +
		".left, " + tableName + ".top, "
		+ tableName + ".color, " + tableName +
		".form, " + tableName + ".finger, " + tableName +
		".status_text FROM " + tableName +
		" WHERE " + tableName + ".key_id = "
		+ QString::number(givenmodifier) + ";")) {
		if (query.first()) {
			*left = query.value(0).toInt();
			*top = query.value(1).toInt();
			*color = query.value(2).toInt();
			*form = query.value(3).toInt();
			*finger = query.value(4).toInt();
			&status->prepend(query.value(5).toString() + " + ");
			return true;
		}
	}
	return false;
}

bool KeyboardSql::getNumLayout(QChar givenchar, int *left, int *top,
	int *color, int *form, int *finger,
	QString *status) {

	// Convert given char into properties of a key
	QString tableName = "num_layouts_" + opSystem;

	QSqlQuery query;
	if (query.exec("SELECT " + tableName +
		".left, " + tableName + ".top, "
		"" + tableName + ".color, " + tableName +
		".form, " + tableName + ".finger, " + tableName +
		".status_text "
		"FROM numpad_list_" + opSystem +
		", " + tableName + " WHERE " + tableName +
		".key_id = numpad_list_" + opSystem +
		".layout AND numpad_list_" + opSystem + ".unicode = " +
		QString::number(givenchar.unicode()) + ";")) {
		if (query.first()) {
			*left = query.value(0).toInt();
			*top = query.value(1).toInt();
			*color = query.value(2).toInt();
			*form = query.value(3).toInt();
			*finger = query.value(4).toInt();
			*status = query.value(5).toString();
			return true;
		}
	}
	return false;
}
